/**************************************************************************
* iptrkg- log grapher (iptrk - ip tracker)
* Copyright (C) 2003 Jason Nunn <jsno@leal.com.au>
* Darwin, NT, Australia.
* Originally developed for Arafura Connect (Dec 2000 - Mar 2003)
* Donated to GPL before liquidation.
*
* =======================================================================
* produces time domain histogram for an IP address
*
**************************************************************************/
#include <math.h>
#include <gd.h>
#include "defs.h"
#include "lib.p"
#include "lib_cgi.p"
#include "iptrkg_defs.h"
#include "iptrkg.p"

typedef struct    /*time domain*/
{
  time_t  time;
  voldr_t voldr;
} td_t;

PRIVATE char   ip_log_path[STR_LEN + 1] = IPTRK_LOG_IP_PATHC,
               addr_str[STR_LEN + 1],
               filename[STR_LEN + 1];
PRIVATE int    addr;
PRIVATE time_t start_time,
               end_time;
PRIVATE ll_t   *td_ll;

/**************************************************************************
*
**************************************************************************/
PRIVATE void load_ip_data(void)
{
  FILE *fp;
  char str[STR_LEN + 1], *s;
  td_t e;

  sprintf(str, "%s/log.%s", ip_log_path, addr_str);
  printf("loading log- %s\n", str);
  fp = fopen(str, "r");
  if(fp == NULL)
  {
    log_e(__func__, "::fopen()");
    exit(-1);
  }

  if(log_ip_fast_find(fp, start_time))
    for(;;)
    {
      if(feof(fp))
        break;

      if(fgets(str, STR_LEN, fp) == NULL)
        break;

      if(str[0] != 'o')
        continue;

      if(sscanf(str, "%*c %lx", &(e.time)) != 1)
        continue;

      if(e.time < start_time || e.time > end_time)
        continue;

      if(fgets(str, STR_LEN, fp) == NULL)
        break;

      s = str;
      if(!get_next_vol_u64(&s, &(e.voldr.I))) continue;
      if(!get_next_vol_u64(&s, &(e.voldr.O))) continue;
      if(!get_next_vol_u64(&s, &(e.voldr.u))) continue;
      if(!get_next_vol_u64(&s, &(e.voldr.i))) continue;
      if(!get_next_vol_u64(&s, &(e.voldr.o))) continue;

      ll_append_unsorted(&td_ll, &e, sizeof(td_t));
    }

  fclose(fp);
}

/**************************************************************************
*
**************************************************************************/
PRIVATE void make_graph(void)
{
  U_64 x;
  int  a1, a2,
       h_pos;
  FILE *fp;
  char str[STR_LEN + 1], str2[STR_LEN + 1], *s;
  ll_t *td_ll_p;
  td_t *td_p;

  void draw_line(U_64 len, int col)
  {
    if(!len)
      return;
    a1 = a2;
    a2 += scale_v(max_vol, BAR_SIZE, len);
    gdImageLine(image, a1, h_pos, a2, h_pos, col);
  }

 /*
  * start
  */
  printf("constructing graph\n");
  no_entries = 0;
  max_vol = 0;
  td_ll_p = td_ll;
  for(;;)
  {
    td_p = ll_read_entry(&td_ll_p);
    if(td_p == NULL)
      break;

    no_entries++;
    x = td_p->voldr.I + td_p->voldr.O +
        td_p->voldr.u +
        td_p->voldr.i + td_p->voldr.o;
    if(x > max_vol)
      max_vol = x;
  }

  graph_init();
  sprintf(str, "iptrk R%s- Time domain histogram", REVISION);
  title_font(image, 15, 5, 20, colour_white, str);

  strncpy(str2, ctime(&start_time), STR_LEN);
  x = strlen(str2);
  if(x > 0)
    str2[x - 1] = 0;
  sprintf(str, "Date range: %s - %s", str2, ctime(&end_time));
  title_font(image, 14, 5, 40, colour_white, str);
  sprintf(str, "Address: %s", addr_str);
  title_font(image, 14, 5, 60, colour_white, str);

  gdImageSetThickness(image, 10);
  h_pos = HIST_Y + 8;
  td_ll_p = td_ll;  
  for(;;)
  {
    td_p = ll_read_entry(&td_ll_p);
    if(td_p == NULL)
      break;

    s = ctime(&(td_p->time));
    s += 3;
    title_font_align(
      image, 14, 0, h_pos + 4, colour_white, s, HIST_X - 5);

    a2 = a1 = BAR_START;
    draw_line(td_p->voldr.I, colour_red);
    draw_line(td_p->voldr.O, colour_blue);
    draw_line(td_p->voldr.u, colour_purple);
    draw_line(td_p->voldr.i, colour_cyan);
    draw_line(td_p->voldr.o, colour_green);
    h_pos += BAR_SPACE;
  }

  sprintf(str, IPTRK_GRAPHS_PATH "/%s", filename);
  fp = fopen(str, "w");
  if(fp != NULL)
  {
    gdImagePng(image, fp);
    fclose(fp);
  }
  else
  {
    log_e(__func__, "::fopen()");
    exit(-1);
  }

  gdImageDestroy(image);
}

/**************************************************************************
*
**************************************************************************/
int main(int argc, char *argv[])
{
  int            x;
  struct in_addr in;
  char           str[STR_LEN + 1];

  welcome();
  log_m = cgi_log_m;
  timer = start_time = end_time = time(NULL);
  for(x = 0; x < argc; x++)
  {
    if(argv[x][0] == '-')
    {
      switch(argv[x][1])
      {
        case 'h':
          printf(
            "Usage: iptrkgt <options>\n"
            " -h                    Display this help info\n"
            " -f <ip_log file>      ip_log's to interrogate\n"
            " -a <address>          IP address\n"
            " -t <time>             Start time (Default = Epoch)\n"
            " -n <time>             End time   (Default = Current time)\n"
            "    (Nb/ Time format can be either YYYY:MM:DD:HH:MM or HH:MM)\n"
            "\n"
          );
          return 0;

        case 'f':
          strncpy2(ip_log_path, parse_cmdline(argc, argv, x), STR_LEN);
          break;

        case 'a':
          inet_aton(parse_cmdline(argc, argv, x), &in);
          addr = ntohl(in.s_addr);
          break;

        case 't':
          start_time = datestr2time(parse_cmdline(argc, argv, x));
          break;

        case 'n':
          end_time = datestr2time(parse_cmdline(argc, argv, x));
          break;
      }
    }
  }

  if(!addr)
  {
    printf("You must specify an IP address.\n");
    exit(-1);
  }

  in.s_addr = htonl(addr);
  strncpy(addr_str, inet_ntoa(in), STR_LEN);
  printf(
    "\n"
    "ip_log path: %s\n"
    "    address: %s\n", 
    ip_log_path, addr_str);
  format_date_entry(start_time, str);
  printf(
    " start time: %s\n", str);
  format_date_entry(end_time, str);
  printf(
    "   end time: %s\n", str);

  delete_old_pngs();

  web_fp = fopen(IPTRK_GRAPHS_PATH "/index.html", "w");
  if(web_fp == NULL)
  {
    log_e(__func__, "::fopen(index.html)");
    exit(-1);
  }

  fprintf(
    web_fp,
    "<html>"
    "<title>iptrk graph stats (Time Domain)</title>"
    "<body><center>"
    "<img src=\"/iptrk.png\"><p>"
    "<font color=\"#80f0a0\">"
    "iptrk graph stats (Time Domain) for %s"
    "<hr>", addr_str);

  sprintf(filename, "%lx.png", random() * timer * getpid());
  load_ip_data();
  make_graph();

  fprintf(
    web_fp,
    "<p><center><img src=\"" IPTRK_GRAPHS_ABS
    "/%s\"></center>", filename);

  fprintf(web_fp, "<hr></center></body></html>\n");
  fclose(web_fp);

  printf("done !\n");

  return 0;
}
